# coding=utf8

import re
import time
import warnings
import requests

try:
    from urlparse import urlparse, parse_qsl  # Python 2
except ImportError:
    from urllib.parse import urlparse, parse_qsl  # Python 3

try:
    import simplejson as json
except ImportError:
    import json


INTERNAL_SERVER_ERROR = 10  # Invalid access token
CAPTCHA_IS_NEEDED = 14


def json_iter_parse(response_text):
    decoder = json.JSONDecoder(strict=False)
    idx = 0
    while idx < len(response_text):
        obj, idx = decoder.raw_decode(response_text, idx)
        yield obj   

class APISession(object):
    def __init__(self, user_phone=None,user_card=None, user_password=None, access_token=None,scope='offline', timeout=1, api_version='0.2'):
        user_login = user_phone or user_card
        if (not user_login or not user_password) and not access_token:
            raise ValueError('Arguments login and user_password, or access_token are required')

        self.user_login = user_login
        self.user_password = user_password

        self.access_token = access_token
        self.scope = scope or ''
        
        self.api_version = api_version

        self._default_timeout = timeout

        self.session = requests.Session()
        self.session.headers = {}
        # self.session.headers['Accept'] = 'application/json'
        # self.session.headers['Content-Type'] = 'application/x-www-form-urlencoded'

        if not access_token and user_login and user_password:
            self.get_access_token()
    def get_access_token(self):

        session = requests.Session()

        oauth_data = {
            'client_id' : "a0fba0c428d9c75c8216",
            'client_secret' : 'b5c41f485819c0b0190ea2415c68b9ce08cae76e',
            'username' : self.user_login,
            'password' : self.user_password,
            'grant_type':'password',
        }
        response = session.post('http://www.kzw.kz/o/access_token', oauth_data)
        if response.status_code != 200:
            try:
                json_data = json.loads(response.text)
            except ValueError:  # not json in response
                error_message = 'KZW OAuth2 grant access error'
            else:
                error_message = 'KZW error: [{0}]'.format(
                    json_data['error']
                    # json_data['error_description']
                )
                raise kzwAuthorizationError(error_message)

        session.close()

        token_dict = json.loads(response.text)
        if 'access_token' in token_dict:
            self.access_token = token_dict['access_token']
            self.expires_in = token_dict['expires_in']
        else:
            raise kzwAuthorizationError('OAuth2 authorization error')
    def __getattr__(self, method_name):
        return APIMethod(self, method_name)

    def __call__(self, method_name, **method_kwargs):
        response = self.method_request(method_name, **method_kwargs)
        response.raise_for_status()

        # there are may be 2 dicts in 1 json
        # for example: {'error': ...}{'response': ...}
        errors = []
        error_codes = []
        for data in json_iter_parse(response.text):
            if 'error' in data:
                error_data = data['error']
                if error_data['error_code'] == CAPTCHA_IS_NEEDED:
                    return self.captcha_is_needed(error_data, method_name, **method_kwargs)

                error_codes.append(error_data['error_code'])
                errors.append(error_data)

            if 'response' in data:
                for error in errors:
                    warnings.warn(str(error))

                # return make_handy(data['response'])
                return data['response']
            
        if INTERNAL_SERVER_ERROR in error_codes:  # invalid access token
            self.get_access_token()
            return self(method_name, **method_kwargs)
        else:
            raise kzwAPIMethodError(errors[0])

    def method_request(self, method_name, timeout=None, **method_kwargs):
        return None
        # if self.access_token:
        #     params = {
        #         'access_token': self.access_token,
        #         'timestamp': int(time.time()),
        #         'v': self.api_version,
        #     }
        #     params.update(method_kwargs)
        #     url = 'http://api.kzw.com/method/' + method_name

        # return self.session.post(url, params, timeout=timeout or self._default_timeout)

    

class APIMethod(object):
    __slots__ = ['_api_session', '_method_name']

    def __init__(self, api_session, method_name):
        self._api_session = api_session
        self._method_name = method_name

    def __getattr__(self, method_name):
        return APIMethod(self._api_session, self._method_name + '.' + method_name)

    def __call__(self, **method_kwargs):
        return self._api_session(self._method_name, **method_kwargs)


class kzwError(Exception):
    pass


class kzwAuthorizationError(kzwError):
    pass


class kzwAPIMethodError(kzwError):
    __slots__ = ['error', 'code']

    def __init__(self, error):
        self.error = error
        self.code = error['error_code']
        super(Exception, self).__init__()

    def __str__(self):
        return "{error_code}. {error_msg}. params = {request_params}".format(**self.error)
